library kuebiko_client;

import 'dart:async';
import 'dart:convert';
import 'dart:typed_data';

import 'package:http/http.dart' as http;
import 'package:kuebiko_client/src/caches/cache_controller.dart';
import 'package:kuebiko_client/src/exception/server_maintenance_exception.dart';
import 'package:kuebiko_client/src/kuebiko_config.dart';
import 'package:kuebiko_client/src/kuebiko_http_client.dart';
import 'package:kuebiko_client/src/models/book.dart';
import 'package:kuebiko_client/src/models/library.dart';
import 'package:kuebiko_client/src/models/mysql_config.dart';
import 'package:kuebiko_client/src/models/series.dart';
import 'package:kuebiko_client/src/models/settings.dart';
import 'package:kuebiko_client/src/models/smtp_config.dart';
import 'package:kuebiko_client/src/models/user.dart';
export 'src/models/user.dart' show User;
export 'src/models/book.dart' show Book, BookSorting, SortingDirection, Formats;
export 'src/kuebiko_config.dart' show KuebikoConfig;
export 'src/models/library.dart' show Library;
export 'src/models/mysql_config.dart' show MysqlConfig;
export 'src/models/series.dart' show Series;
export 'src/models/settings.dart' show Settings;
export 'src/models/smtp_config.dart' show SmtpConfig;
export 'src/models/progress.dart' show Progress;
export 'src/models/task.dart' show Task;

class KuebikoClient {

  final KuebikoConfig _config;
  late final KuebikoHttpClient _client;
  late final CacheController _cacheController;
  late final Settings _settings;
  bool _initialized = false;

  KuebikoClient(this._config){
    _client = KuebikoHttpClient(_config, http.Client());
    _settings = Settings(_client);
    _cacheController = CacheController(_client);

    _initialize();
  }

  _initialize() async {
    if(this._config.apiKey != null){
      await _cacheController.getLibraryCache().update();
      await _cacheController.getSeriesCache().update();
      this._initialized = true;
    }
  }

  // Creates an API token with the login credentials and returns a Client object
  static Future<KuebikoClient> login(KuebikoConfig config, String username, String password) async
  {
    try {
      KuebikoClient tmpClient = KuebikoClient(config);
      if(await tmpClient.status() != 'Running'){
        throw ServerMaintenanceModeException();
      }
    } catch(e){
      rethrow;
    }

    Uri loginUri = config.generateApiUri('/api-key');
    String encodedCredentials = base64.encode((username+':'+password).codeUnits);
    http.Response res = await http.get(
        loginUri,
        headers: {
          'Authorization': 'Basic ' + encodedCredentials
        }
    );
    Map resJson = jsonDecode(res.body);
    config = KuebikoConfig(
        appName: config.appName,
        appVersion: config.appVersion,
        baseUrl: config.baseUrl,
        deviceName: config.deviceName,
        apiKey: resJson['token']
    );
    return KuebikoClient(config);
  }

  // Returns the Kuebiko Config
  KuebikoConfig getConfig() => this._config;
  bool getInitialized() => this._initialized;

  Future<Book> upload(Library library, Uint8List fileContent) async
  => await Book.upload(library, _cacheController, _client, fileContent);

  Future<List<Book>> getBooks(BookSorting sorting, SortingDirection sortingDirection) async {
    await _checkInitialized();

    List<Book> books = await Book.getBooks(sorting, sortingDirection, _cacheController, _client);
    return books;
  }

  _checkInitialized() async {
    final Duration waitDuration = Duration(milliseconds: 200);
    while(!this._initialized){
      await Future.delayed(waitDuration);
    }
  }

  Future<List<Library>> getLibraries() async {
    await _checkInitialized();
    return _cacheController.getLibraryCache().getAll();
  }

  Future<Library> createLibrary(String name, String path) => Library.create(name, path, _cacheController, _client);

  Future<void> createFolder(String path) => Library.createFolder(path, _client);
  Future<List<String>> getFolderContent(String path) => Library.getFolderContent(path, _client);

  void scanAll() => Library.scanAll(_client);

  void renewMetadataAll() => Library.renewMetadataAll(_client);

  Future<Series> createSeries({
    required String name,
    required String author,
    required String description,
    required int numberOfVolumes,
    required String publisher,
    required String language,
    required String genre,
    required String ageRating,
    required String type,
    required List<String> locked
  }) => Series.create(
    name: name,
    author: author,
    description: description,
    numberOfVolumes: numberOfVolumes,
    publisher: publisher,
    language: language,
    genre: genre,
    ageRating: ageRating,
    type: type,
    locked: locked,
    cacheController: _cacheController,
    httpClient: _client,
  );

  Future<List<Series>> getAllSeries() async {
    await _checkInitialized();
    return this._cacheController.getSeriesCache().getAll();
  }

  Future<String> status() => Settings.status(_client);

  Future<Uri> docs() => Settings.docs(_client);

  Settings getSettings() => _settings;

  Future<User> currentUser() => User.currentUser(_client, _cacheController);

  Future<List<User>> getUsers() => User.getAll(_client, _cacheController);

  Future<User> createUser(
      String email,
      String name,
      String password,
      List<String> role,
      String anilistName,
      String anilistToken
      ) => User.create(
      email,
      name,
      password,
      role,
      anilistName,
      anilistToken,
      _client,
      _cacheController
  );
}


Future<void> setup({
  required KuebikoConfig config,
  required SmtpConfig smtpConfig,
  required MysqlConfig mysqlConfig,
  required int scanInterval,
  required Uri url,
  required String anilistToken,
  required String adminUsername,
  required String adminEmail,
  required String adminPassword
}) async {

  KuebikoHttpClient httpClient = KuebikoHttpClient(config, http.Client());
  CacheController cacheController = CacheController(httpClient);
  User adminUser = User(
      0,
      adminUsername,
      adminEmail,
      ['admin'],
      httpClient,
      cacheController
  );
  return await Settings.setup(
      httpClient: httpClient,
      smtpConfig: smtpConfig,
      mysqlConfig: mysqlConfig,
      scanInterval: scanInterval,
      url: url,
      anilistToken: anilistToken,
      adminUser: adminUser,
      adminPassword: adminPassword
  );
}