import 'dart:async';
import 'dart:convert';
import 'dart:typed_data';

import 'package:image/image.dart';
import 'package:kuebiko_client/src/caches/cache_controller.dart';
import 'package:kuebiko_client/src/kuebiko_multipart_request.dart';
import 'package:kuebiko_client/src/models/library.dart';
import 'package:kuebiko_client/src/kuebiko_http_client.dart';
import 'package:http/http.dart' as http;
import 'package:kuebiko_client/src/models/progress.dart';

enum BookSorting {
  name,
  upload,
  published,
  author,
  number
}

enum SortingDirection {
  asc,
  desc
}

enum Formats {
  epub,
  ebz,
  pdf,
  azw3
}

class Book {
  final int id;
  String name;
  Library _library;
  final CacheController _cacheController;

  final KuebikoHttpClient _httpClient;

  Book(this.id, this._library, this.name, this._cacheController, this._httpClient);


  static Future<Book> upload(Library library, CacheController cacheController, KuebikoHttpClient httpClient, Uint8List fileContent) async {
    http.MultipartRequest req = KuebikoMultipartRequest(
        'POST',
        httpClient,
        httpClient.config.generateApiUri('/upload')
    );

    req.files.add(
        http.MultipartFile.fromBytes('book', fileContent)
    );
    req.fields['library'] = library.id.toString();

    http.StreamedResponse res = await req.send();
    List<int> responseMergedRaw = [];
    List<List<int>> responseRaw = await res.stream.toList();
    responseRaw.forEach((List<int> list) => responseMergedRaw.addAll(list));
    String responseContent = utf8.decode(responseMergedRaw);
    Map jsonContent = jsonDecode(responseContent);
    Book book = Book(jsonContent['bookId'], library, jsonContent['bookname'], cacheController, httpClient);
    return book;
  }

  static getBooks(BookSorting sort, SortingDirection direction, CacheController cacheController, KuebikoHttpClient httpClient) async {
    Uri uri = httpClient.config.generateApiUri(
        '/books',
        queryParameters: {
          'sort': sort.toString(),
          'direction': direction.toString()
        }
    );
    http.Response res = await httpClient.get(uri);
    Map json = jsonDecode(res.body);
    List bookRaw = json['books'];

    List<Book> books = [];

    for(int i = 0; i < bookRaw.length; i++){
      Map book = bookRaw[i];
      Library library = await cacheController.getLibraryCache().getById(book['library']);
      books.add(
          Book(book['id'], library, book['name'], cacheController, httpClient));
    }

    return books;
  }

  Future<Uint8List> download(Formats format) async {
    // TODO test method
    Uri uri = this._httpClient.config.generateApiUri(
        '/download',
        queryParameters: {
          'library': this._library.id,
          'book': this.id,
          'format': format.toString()
        }
    );
    http.Response res = await this._httpClient.get(uri);

    return res.bodyBytes;
  }

  Future<String> convert(Formats format) async {
    Uri uri = this._httpClient.config.generateApiUri(
        '/convert',
        queryParameters: {
          'library': _library.id.toString(),
          'book': this.id.toString(),
          'format': format.toString()
        }
    );
    http.Response res = await _httpClient.put(uri);
    return jsonDecode(res.body)['convertId'];
  }

  Future<String> convertStatus(String convertId) async {
    Uri uri = _httpClient.config.generateApiUri(
        '/convert/status',
        queryParameters: {
          'convertId': convertId
        }
    );
    http.Response res = await _httpClient.get(uri);
    return jsonDecode(res.body)['status'];
  }

  void delete() async {
    Uri uri = _httpClient.config.generateApiUri(
        '/delete',
        queryParameters: {
          'library': _library.id.toString(),
          'book': this.id.toString()
        }
    );
    _httpClient.delete(uri);
  }

  Future<Map> metadata() async{
    Uri uri = _httpClient.config.generateApiUri(
        '/metadata',
        queryParameters: {
          'library': _library.id.toString(),
          'book': id.toString()
        }
    );
    http.Response res = await _httpClient.get(uri);
    Map metadata = jsonDecode(res.body)['book'];
    metadata['series'] = await _cacheController.getSeriesCache().getById(metadata['series']['id']);
    return metadata;
  }

  void renewMetadata() {
    Uri uri = _httpClient.config.generateApiUri(
        '/book/metadata',
        queryParameters: {
          'library': _library.id.toString(),
          'book': id.toString()
        }
    );

    _httpClient.put(uri);
  }

  Future<Image?> cover() async{
    Uri uri = this._httpClient.config.generateApiUri(
        '/cover',
        queryParameters: {
          'library': _library.id.toString(),
          'book': this.id.toString()
        }
    );

    http.Response res = await _httpClient.get(uri);
    return decodeImage(res.bodyBytes);
  }

  Future<void> setProgress(Progress progress) async {
    Uri uri = this._httpClient.config.generateApiUri(
        '/v1/' + _library.id.toString() + '/' + this.id.toString() + '/progress',
      queryParameters: {
          'reading_page': progress.currentPage.toString()
      }
    );
    await _httpClient.put(uri);
  }

  Future<Progress> getProgress() async {
    Uri uri = this._httpClient.config.generateApiUri(
        '/' + _library.id.toString() + '/' + this.id.toString() + '/progress'
    );
    http.Response res = await _httpClient.get(uri);
    Map json = jsonDecode(res.body);
    if(json['reading_page'] is int){
      json['reading_page'] = json['reading_page'].toDouble();
    }
    return Progress(currentPage: json['reading_page'], maxPage: json['max_page']);
  }
}