import 'dart:convert';

import 'package:kuebiko_client/src/caches/cache_controller.dart';
import 'package:kuebiko_client/src/kuebiko_http_client.dart';
import 'package:http/http.dart' as http;
import 'package:kuebiko_client/src/models/book.dart';

class User {
  final int id;
  String _name;
  String _email;
  List<String> _role;
  List<String> _changed = [];
  final KuebikoHttpClient _httpClient;
  final CacheController _cacheController;

  User(this.id, this._name, this._email, this._role, this._httpClient, this._cacheController);

  static Future<List<User>> getAll(KuebikoHttpClient httpClient, CacheController cacheController) async {
    Uri uri = httpClient.config.generateApiUri('/users');
    http.Response res = await httpClient.get(uri);
    Map json = jsonDecode(res.body);
    List usersRaw = json['users'];
    return usersRaw.map((user) => User(
        user['id'],
        user['name'],
        user['email'],
        user['role'].cast<String>(),
        httpClient,
        cacheController
    )).toList();
  }

  static Future<User> currentUser(KuebikoHttpClient httpClient, CacheController cacheController) async {
    Uri uri = httpClient.config.generateApiUri('/user');
    http.Response res = await httpClient.get(uri);
    Map json = jsonDecode(res.body);
    Map userRaw = json['user'];
    return User(
        userRaw['id'],
        userRaw['name'],
        userRaw['email'],
        userRaw['role'].cast<String>(),
        httpClient,
        cacheController
    );
  }

  static Future<User> create(
      String email,
      String name,
      String password,
      List<String> role,
      String anilistName,
      String anilistToken,
      KuebikoHttpClient httpClient,
      CacheController cacheController
      ) async {
    Uri uri = httpClient.config.generateApiUri('/user/create');
    http.Response res = await httpClient.post(
        uri,
        body: {
          'email': email,
          'name': name,
          'password': password,
          'role': role.join(','),
          'anilist': jsonEncode({
            'name': anilistName,
            'token': anilistToken
          })
        }
    );
    Map json = jsonDecode(res.body);
    return User(json['user'], name, email, role, httpClient, cacheController);
  }

  void update(String password) {
    Uri uri = _httpClient.config.generateApiUri('/user/edit');

    Map data = {
      'password': password
    };
    _changed.forEach((String changedElement) {
      switch(changedElement){
        case 'name':
          data['name'] = _name;
          break;
        case 'email':
          data['email'] = _email;
          break;
        case 'role':
          data['role'] = _role;
          break;
      }
    });

    _httpClient.put(
        uri,
        body: data
    );
  }

  void delete(String password) async {
    Uri uri = _httpClient.config.generateApiUri('/user/delete');
    await _httpClient.delete(
        uri,
        body: {
          'password': password
        }
    );
  }

  void tokenDelete(int tokenId){
    Uri uri = _httpClient.config.generateApiUri(
        '/token/delete',
        queryParameters: {
          'tokenid': tokenId.toString()
        }
    );
    _httpClient.delete(uri);
  }

  Future<List<Book>> unreadBooks() async {
    Uri uri = _httpClient.config.generateApiUri('/unread');
    http.Response res = await _httpClient.get(uri);
    Map json = jsonDecode(res.body);
    List<Book> books = [];

    for(int i = 0;i < json['books'].length; i++){
      Map book = json['books'][i];
      books.add(
          Book(
              book['id'],
              await this._cacheController.getLibraryCache().getById(book['library']),
              book['name'],
              _cacheController,
              _httpClient
          )
      );
    }
    return books;
  }

  Future<List<Book>> readingBooks() async {
    Uri uri = _httpClient.config.generateApiUri('/reading');
    http.Response res = await _httpClient.get(uri);
    Map json = jsonDecode(res.body);
    List<Book> books = [];

    for(int i = 0;i < json['books'].length; i++){
      Map book = json['books'][i];
      books.add(
          Book(
              book['id'],
              await this._cacheController.getLibraryCache().getById(book['library']),
              book['name'],
              _cacheController,
              _httpClient
          )
      );
    }
    return books;
  }

  Future<List<Book>> finishedBooks() async {
    Uri uri = _httpClient.config.generateApiUri('/finished');
    http.Response res = await _httpClient.get(uri);
    Map json = jsonDecode(res.body);
    List<Book> books = [];

    for(int i = 0;i < json['books'].length; i++){
      Map book = json['books'][i];
      books.add(
          Book(
              book['id'],
              await this._cacheController.getLibraryCache().getById(book['library']),
              book['name'],
              _cacheController,
              _httpClient
          )
      );
    }
    return books;
  }
  String getEmail() => _email;
  String getName() => _name;
  List<String> getRoles() => _role;
}